#define E1 9
#define A1 5
#define B1 6
#define E2 10
#define A2 7
#define B2 8

#include <Arduino_BMI270_BMM150.h>

// PID coefficients
float Kp = 0.1;
float Ki = 0;
float Kd = 0.03;

// PID variables
float pid_error = 0;
float previous_error = 0;
float integral = 0;
float derivative = 0;
float output = 0;

float setpoint = 7;
float angle_limite = 6;

float pitch = 0.0;
float accAngle = 0.0;
float gyroRate = 0.0;
float alpha = 0.85;  // Poids du gyroscope

unsigned long lastTime = 0;
unsigned long lastControlTime = 0;
const int controlInterval = 5; // ms, 100 Hz

void setup() {
  pinMode(E1, OUTPUT);
  pinMode(E2, OUTPUT);
  pinMode(A1, OUTPUT);
  pinMode(B1, OUTPUT);
  pinMode(A2, OUTPUT);
  pinMode(B2, OUTPUT);

  Serial.begin(9600);
  //while (!Serial);

  if (!IMU.begin()) {
    Serial.println("IMU non détecté !");
    // Ne bloque plus, juste un message
  } else {
    Serial.println("IMU initialisé !");
  }
  delay(100);
  lastTime = millis();
}

void loop() {
  unsigned long now = millis();
  
  // Contrôle PID toutes les 10 ms
  if (now - lastControlTime >= controlInterval) {
    lastControlTime = now;

    float ax, ay, az;
    float gx, gy, gz;

    if (IMU.accelerationAvailable() &&
        IMU.gyroscopeAvailable()) {
      IMU.readAcceleration(ax, ay, az);
      IMU.readGyroscope(gx, gy, gz);
    }

    // 1. Calcul de l'angle avec l'accéleromètre
    accAngle = atan2(ax, sqrt(ay*ay + az*az)) * 180.0 / PI;
    // 2. Vitesse angulaire autour de l’axe X (par défaut en deg/s)
    gyroRate = gx;

    // PID
    float dt = (now - lastTime) / 1000.0;
    lastTime = now;

    // 3. Filtre complémentaire
    pitch = alpha * (pitch + gyroRate * dt) + (1 - alpha) * accAngle;

    pid_error = setpoint - pitch;
    integral += pid_error * dt;
    derivative = (pid_error - previous_error) / dt;
    output = Kp * pid_error + Ki * integral + Kd * derivative;
    previous_error = pid_error;

    // Sortie normalisée entre -1 et 1
    output = -constrain(output, -1.0, 1.0);

    if(pid_error>angle_limite){
      output = -1;
    }
    if(pid_error<-angle_limite){
      output = 1;
    }

    // Durée d’activation en ms dans cette période de 10 ms
    int onTime = (int)(abs(output) * controlInterval);
    int offTime = controlInterval - onTime;


    // Direction des moteurs
    if (output > 0) {
      digitalWrite(A1, HIGH);
      digitalWrite(B1, LOW);
      digitalWrite(A2, HIGH);
      digitalWrite(B2, LOW);
    } else if (output < 0) {
      digitalWrite(A1, LOW);
      digitalWrite(B1, HIGH);
      digitalWrite(A2, LOW);
      digitalWrite(B2, HIGH);
    } else {
      digitalWrite(A1, LOW);
      digitalWrite(B1, LOW);
      digitalWrite(A2, LOW);
      digitalWrite(B2, LOW);
    }

    // Activer moteurs pendant onTime
    digitalWrite(E1, HIGH);
    digitalWrite(E2, HIGH);
    delay(onTime);

    // Éteindre moteurs pendant offTime
    digitalWrite(E1, LOW);
    digitalWrite(E2, LOW);
    delay(offTime);

    // Debug
    Serial.print("Pitch: ");
    Serial.print(pitch);
    Serial.print(" | Out: ");
    Serial.print(output, 2);
    Serial.print(" | onTime: ");
    Serial.println(onTime);
  }
}
















